import 'package:flutter/material.dart';
import 'package:modular_system_core/modular_system_core.dart';

class EntityUpdateForm {
  EntityUpdateForm({
    required this.entity,
    required this.controller,
  });

  AppEntity entity;
  FormController controller;

  EntityTextUpdateField textUpdateField({
    bool? forbidSpaces,
    required String id,
    required String updateName,
    required String hintText,
    required String currentValue,
    RegExp? forbiddenCharacters,
  }) {
    return EntityTextUpdateField(
      forbidSpaces: forbidSpaces,
      id: id,
      updateName: updateName,
      hintText: hintText,
      currentValue: currentValue,
      formController: controller,
      entity: entity,
      forbiddenCharacters: forbiddenCharacters,
    );
  }
}

class UpdateForm {
  UpdateForm({
    required this.controller,
  });

  FormController controller;

  TextUpdateField textUpdateField({
    bool? forbidSpaces,
    required String hintText,
    required String currentValue,
    required Future<dynamic> Function(String, Event) submitCallback,
    RegExp? forbiddenCharacters,
  }) {
    return TextUpdateField(
      forbidSpaces: forbidSpaces,
      hintText: hintText,
      currentValue: currentValue,
      controller: controller,
      submitCallback: submitCallback,
      forbiddenCharacters: forbiddenCharacters,
    );
  }
}

class TextUpdateField extends StatefulWidget {
  // ignore: use_key_in_widget_constructors
  const TextUpdateField({
    this.forbidSpaces,
    required this.hintText,
    required this.currentValue,
    required this.controller,
    this.fieldController,
    required this.submitCallback,
    this.forbiddenCharacters,
  });

  final bool? forbidSpaces;
  final String hintText;
  final String currentValue;
  final FormController controller;
  final FieldController? fieldController;
  final Future Function(String value, Event success) submitCallback;
  final RegExp? forbiddenCharacters;

  @override
  State<TextUpdateField> createState() {
    // ignore: no_logic_in_create_state
    return _TextUpdateFieldState(
      hintText: hintText,
      currentValue: currentValue,
      formController: controller,
      fieldController: fieldController,
      submitCallback: submitCallback,
    );
  }
}

class _TextUpdateFieldState extends State<TextUpdateField> {
  _TextUpdateFieldState({
    this.forbidSpaces,
    required this.hintText,
    required this.currentValue,
    required this.formController,
    this.fieldController,
    required this.submitCallback,
    this.forbiddenCharacters,
  }) {
    subscribeToEvents();
    fieldController ??= FieldController();
  }

  // injected
  final bool? forbidSpaces;
  final String hintText;
  final FormController formController;
  late FieldController? fieldController;
  final RegExp? forbiddenCharacters;
  late AppForm form = AppForm(controller: formController);
  // state
  bool isEditMode = false;
  String currentValue;
  String editorValue = '';
  Event successEvent = Event();
  late Future submitFuture = Future(() {});
  Future Function(String value, Event success) submitCallback;

  @override
  void dispose() {
    unsubscribeToEvents();
    super.dispose();
  }

  void subscribeToEvents() {
    successEvent.subscribe(onSuccess);
  }

  void unsubscribeToEvents() {
    successEvent.unsubscribe(onSuccess);
  }

  @override
  Widget build(BuildContext context) {
    return ExpansionPanelList(
      children: [
        ExpansionPanel(
          isExpanded: isEditMode,
          canTapOnHeader: true,
          headerBuilder: (context, isExpanded) {
            return Padding(
              padding: const EdgeInsets.symmetric(horizontal: 15),
              child: Row(
                children: [
                  Icon(
                    Icons.edit,
                    size: formController.viewModel.textSize.data,
                    color: formController.viewModel.colorScheme.onBackground,
                  ),
                  const VerticalDivider(),
                  Text(
                    "Valor Original: $currentValue",
                    style: formController.viewModel.backgroundTextStyle(),
                  ),
                ],
              ),
            );
          },
          body: Container(
            padding: EdgeInsets.all(formController.viewModel.textSize.data),
            decoration: BoxDecoration(
              border: Border(
                top: BorderSide.none,
                bottom: BorderSide(
                  color: formController.viewModel.colorScheme.onBackground,
                ),
                right: BorderSide(
                  color: formController.viewModel.colorScheme.onBackground,
                ),
                left: BorderSide(
                  color: formController.viewModel.colorScheme.onBackground,
                ),
              ),
            ),
            child: Column(
              children: [
                form.textField(
                  forbidSpaces: forbidSpaces ?? false,
                  hintText: hintText,
                  fieldController: fieldController,
                  forbiddenCharacters: forbiddenCharacters,
                  getValue: () => editorValue,
                  setValue: (value) => editorValue = value,
                ),
                Row(
                  mainAxisAlignment: MainAxisAlignment.end,
                  children: [
                    FutureBuilder(
                      future: submitFuture,
                      builder: (context, snapshot) {
                        if (snapshot.connectionState == ConnectionState.done) {
                          return OutlinedButton(
                            style: OutlinedButton.styleFrom(
                              padding: EdgeInsets.all(
                                formController.viewModel.textSize.data,
                              ),
                            ),
                            child: Text(
                              "Atualizar",
                              style: formController.viewModel
                                  .backgroundTextStyle(),
                            ),
                            onPressed: onSubmit,
                          );
                        }
                        return CircularProgressIndicator(
                          color: formController.viewModel.colorScheme.tertiary,
                        );
                      },
                    ),
                    VerticalDivider(
                      width: formController.viewModel.textSize.data,
                    ),
                    OutlinedButton(
                      style: OutlinedButton.styleFrom(
                        padding: EdgeInsets.all(
                          formController.viewModel.textSize.data,
                        ),
                      ),
                      child: Text(
                        "Cancelar",
                        style: formController.viewModel.backgroundTextStyle(),
                      ),
                      onPressed: () {
                        setState(() {
                          isEditMode = false;
                        });
                        fieldController!.resetEvent.invoke();
                      },
                    ),
                  ],
                )
              ],
            ),
          ),
        ),
      ],
      expansionCallback: (index, isOpen) {
        setState(() {
          isEditMode = !isEditMode;
        });
      },
    );
  }

  void onSubmit() {
    setState(() {
      submitFuture = Future(() async {
        try {
          if (!formController.getIsFormValid()) {
            return;
          }
          await submitCallback(editorValue, successEvent);
        } catch (e) {
          formController.viewModel.appEvents.errorEvent.invoke(e);
        }
      });
    });
  }

  void onSuccess() {
    setState(() {
      currentValue = editorValue;
      isEditMode = false;
    });
    fieldController!.resetEvent.invoke();
  }
}

class EntityTextUpdateField extends StatelessWidget {
  // ignore: use_key_in_widget_constructors
  EntityTextUpdateField({
    this.forbidSpaces,
    required this.id,
    required this.updateName,
    required this.hintText,
    required this.currentValue,
    required this.formController,
    required this.entity,
    this.forbiddenCharacters,
  });

  final bool? forbidSpaces;
  final String id;
  final String updateName;
  final String hintText;
  final String currentValue;
  final FormController formController;
  final FieldController fieldController = FieldController();
  final AppEntity entity;
  final RegExp? forbiddenCharacters;

  @override
  Widget build(BuildContext context) {
    return TextUpdateField(
      forbidSpaces: forbidSpaces,
      hintText: hintText,
      currentValue: currentValue,
      controller: formController,
      fieldController: fieldController,
      submitCallback: submitAsync,
      forbiddenCharacters: forbiddenCharacters,
    );
  }

  Future<dynamic> submitAsync(String value, Event success) async {
    await entity.controller.updateAsync(getRequest());
    success.invoke();
  }

  WebUpdateRequest getRequest() {
    var req = WebUpdateRequest(id: id);
    var update = WebUpdateField();
    update.isActive = true;
    update.name = updateName;
    update.type = WebDataType.string;
    update.value = fieldController.getValueEvent.invoke();
    req.updates.add(update);
    return req;
  }
}
